package nl.uvt.locator;

import java.util.List;
import java.util.Properties;
import nu.xom.Document;

public class TucatConnection {

	List<LinkInfo> query( QueryData pQ ) throws LocatorException {
		List<LinkInfo> meResult = null;
		String url = Locator.locatorConfig.getProperty("tucatUri");
		String query = "";
		try {
			Properties prop = new Properties( Locator.sruDefaultProps );
			prop.setProperty("recordSchema","picamarc" );
			if ( !pQ.epn.equals("") ) {
				// Thomas would like a direct access to availability here
				// but don't we need stuff like collection, and callNumber?
				Locator.logger.debug("direct search with EPN: " + pQ.epn );
				query = "pica.epn=" + pQ.epn;
			}
			else if ( pQ.genre.equals( "article") ){
				if ( pQ.issn.length() > 0 )
					query = "pica.num=" + pQ.issn;
				else if ( pQ.title.length() > 0 ){
					String title = pQ.title;
					String titleParts[] = title.split( ":", 2 );
					if ( titleParts.length > 1 ){
						for( int i=0; i < titleParts.length; ++i )
							Locator.logger.debug( "part["+i+"]= '"+titleParts[i] +"'");
								title = titleParts[0];
					}
					// we don't want spurious " in our query (tucat seem to swallow it, but it is invalid CQL)
					title = title.replaceAll("\"", "");
					// index is case insensitive and stopwords are assumed lowercase. 
					title = title.toLowerCase();
					title = MerescoConnection.filterStopWords( title );
					query = "pica.htl=\"" + title + "\"";
				}
			}
			else if ( pQ.isbn.length() > 0 ){
				Locator.logger.debug("ISBN: " + pQ.isbn );
				if ( pQ.isbn.length() > 10 ){
					Locator.logger.debug("ISBN: is lang genoeg " + pQ.isbn.length() );
					query = "pica.num=" + pQ.isbn;
				}
				else {
					Locator.logger.debug("ISBN verlengen " );
					String isbn13 = to13( pQ.isbn );
					query = "pica.num=" + pQ.isbn + " or pica.num=" + isbn13;						
				}
			}
			else if ( pQ.location.length() > 0 ){
				String location = picaFixLocation( pQ.location );
				query = "pica.sig=\"" + location + "\"";
			}
			else {
				String title = null;
				if ( pQ.atitle.length() > 0 ) {
					title = picaFixTitle( pQ.atitle );
				}
				else if ( pQ.title.length() > 0 ) {
					title = picaFixTitle( pQ.title );
				}
				String author = null;
				if ( !pQ.genre.equals("bookitem") && 	
						!pQ.genre.equals("proceeding") &&
						pQ.aulast.length() > 0 ){
					author = "pica.aut=\"" + pQ.aulast + "\""; 
				}
				if ( title != null ) {
					query = "pica.ttl=\"" + title + "\"";
					if ( author != null )
						query += " and " + author;
				}
				else if ( author != null )
					query = author;
			}
			if ( query.length() == 0 )
				// fails when no usefull data is available
				return meResult;
			prop.setProperty("query", query );
			prop.setProperty("sortKeys","YOP,pica,0" );
			url = HttpConnection.createUrlString(url, prop );
			Locator.logger.debug("Try URL: " + url );
			Document result = HttpConnection.httpGetDocument( url, Locator.httpConfig );
			if ( result != null ){
				int hits = Locator.extractHits( result );
				Locator.logger.debug("tucat numOfRecords=" + hits );
				if ( hits > 0 ){
//					Locator.logger.debug( "tucat doc:" + result.toXML());
					String sheetName = Locator.locatorConfig.getProperty("tucatSheet");
					meResult = HoldingInfo.getModsResults( result, pQ, sheetName );
				}
			}
			else {
				Locator.logger.warn("No response from:" + url );
			}
		}	
		catch ( Exception e) {
			throw new LocatorException( "Tucat URL: '" + url + "' failed"); 
		}
		return meResult;
	}

	private String picaFixLocation( String loc ) {
		String result = loc.replaceAll("/\\d{4}.*", "/...");
		if ( !loc.equals(result))
			Locator.logger.debug("piacaFixLocation replaced '" + loc + "' by: '" + result + "'" );			
		return result;
	}

	private String picaFixTitle (String title) {
		// pica supports only strings of 64 chars, it seems
		String result = title;
		result = result.replace('"', ' ');
		if ( title.length() > 64 ){
			result = result.replace('(', ' ');
			result = result.replace(')', ' ');
			result = result.replace('[', ' ');
			result = result.replace(']', ' ');
			result = result.replace('{', ' ');
			result = result.replace('}', ' ');
			int pos = result.lastIndexOf(' ');
			while ( pos > 0 && pos > 64 ){
				pos = result.lastIndexOf(' ', pos-1 );
			}
			if ( pos == 0 ){
				pos = result.indexOf(':');
			}
			if ( pos > 0 )
				result = result.substring(0,pos);
			Locator.logger.debug("fixed up title from '" + title + "' to '" + result + "'");
		}
		return result;
	}

	private String to13(String isbn) {
		Locator.logger.debug( "ISBN10to13 :"+ isbn );
		if ( isbn.length() < 10 ){
			Locator.logger.debug( "isbn is to short!");
			return isbn;
		}
		String NUMVALUES = "0123456789";
		String work = isbn.substring(0, 9);
		work = "978" + work;
		char[] bytes = work.toCharArray();
		boolean three = false;
		int checksum = 0;
		for ( int i=0; i < 12; ++i ){
			if ( three )
				checksum += 3 * NUMVALUES.indexOf(bytes[i]);
			else
				checksum += 1 * NUMVALUES.indexOf(bytes[i]);
			three = !three;
		}
		int rem = checksum % 10;
		int last = 0;
		if ( rem != 0 )
			last = 10 - rem;
		String result = new String(bytes) + last;
		Locator.logger.debug( "ISBN10to13 gives "+ result );
		return result;
	}


}
